package internal

import (
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"gitea.cdlsxd.cn/sdk/plugin/proto"
	"github.com/carlmjohnson/requests"
	"plugins/zltx_card/internal/po"
)

// 插件通信信息,若不对应则会报错panic
const (
	Tag         = "zltx_card"
	Version     = 1
	CookieKey   = "zltx_card"
	CookieValue = "zltx_card"
)

const (
	orderMethod = "/card/order"
	queryMethod = "/card/query"
)

type ZLTXCardService struct{}

func (p *ZLTXCardService) Order(ctx context.Context, request *proto.OrderRequest) (*proto.OrderResponse, error) {
	c, err := transConfig(request.Config)
	if err != nil {
		return nil, err
	}
	uv, err := req(c.orderReq(request), c.AppKey)
	if err != nil {
		return nil, err
	}

	var response po.OrderResp
	baseUrl := fmt.Sprintf("%s%s", c.BaseUri, orderMethod)
	err = requests.URL(baseUrl).BodyForm(uv).ToJSON(&response).Fetch(ctx)
	if err != nil {
		return nil, fmt.Errorf("请求异常，msg:" + err.Error())
	}

	response.SetCodeStr()
	if !response.CodeStr.IsRechargeSuccess() {
		return nil, fmt.Errorf("请求错误,msg:" + response.Message)
	}

	return orderResp(request, response), nil
}

func (p *ZLTXCardService) Query(ctx context.Context, request *proto.QueryRequest) (*proto.QueryResponse, error) {
	c, err := transConfig(request.Config)
	if err != nil {
		return nil, err
	}
	uv, err := req(c.queryReq(request), c.AppKey)
	if err != nil {
		return nil, err
	}

	var response po.QueryResp
	err = requests.URL(c.BaseUri + queryMethod).BodyForm(uv).ToJSON(&response).Fetch(ctx)
	if err != nil {
		return nil, fmt.Errorf("请求异常，msg:" + err.Error())
	}

	if !response.Code.IsSuccess() {
		return nil, errors.New("请求错误，msg:" + response.Message)
	}

	return queryResp(request, response, c.AppKey)
}

func (p *ZLTXCardService) Notify(_ context.Context, request *proto.NotifyRequest) (*proto.NotifyResponse, error) {
	c, err := transConfig(request.Config)
	if err != nil {
		return nil, err
	}
	var poReq po.Notify
	err = json.Unmarshal(request.Body, &poReq)
	if err != nil {
		return nil, fmt.Errorf("解析异常，msg:" + err.Error())
	}

	err = poReq.Validate()
	if err != nil {
		return nil, err
	}
	if !verify(&poReq, c.AppKey) {
		return nil, fmt.Errorf("验签不通过")
	}

	return notifyResp(poReq, c.AppKey)
}
